<?php
//TITLE=SND file v3 handler (NES music/S3M/MIDI converter)

$title = 'SND file v3 handler';
$progname = 'sndtool2';

$text = array(
   'purpose:1. Purpose' => "

Read SND file, play it, convert it to S3M or MIDI.

", 'format:1. SND file format' => "

SND is a file format for storing NES
(and some other console) music (almost) losslessly.

<pre>".htmlspecialchars(join('',file('/home/bisqwit/src/sndtool2/sndform.txt')))."</pre>

The program converts the SND to MidiS3M with Adlib instruments.<br>
PCM samples are converted into S3M digital samples.<br>
Conversion to MIDI format is also supported, but it
doesn't save the PCM samples.

", 'examples:1. Examples' => "

<a href=\"/src/arch/sndtool2-examples.zip\">This zip file</a> contains
some S3M files generated by sndtool2 (only the instrument list
has been finetuned by hand) and one MIDI generated by sndtool2
(not edited).<br>
Note that the S3M files use adlib instruments.<br>
Most mod players are unable to playback them, and most
of the rest which can, only use an emulator which isn't
perfect.

", 'status:1. Development status' => "

The program works, but isn't yet ready for users.<br>
Command line options have to be implemented and the instrument type heuristics enhanced.<br>
Three primary goals are:
<ul>
 <li>To get the required amount of manual finetuning work to minimum.</li>
 <li>To produce as exact conversion as possible, without hindering the next:</li>
 <li>To produce as readable and editable conversion as
  possible. (To allow easily copying the great melodies.)</li>
</ul>

", '1. How to use' => "

<ol>
 <li>First you have a SND file. You could create it with <code>nezplay</code>
by giving it the <code>-l</code> option on commandline (press <code>L</code>
to stop logging and save).<br>
 <li>Edit <code>instrument.cc</code> from sndtool2. Find the <code>GM</code>
  and <code>GP</code> parts, edit the numbers as you want.
   The numbers correspond to General MIDI instrument numbers,
   for example GM(1) is acoustic grand piano. You can find the
   complete list of instruments in <code>midi-gm.c</code>.</li>
 <li>Recompile: <code>make snd</code></li>
 <li>Run: <code>./snd  sndfilename.snd  >/dev/null 2> tmptmp
  <ul>
   <li>Short version: <code>./snd  sndfilename.snd &> /dev/null </code></li>
  </ul></li>
 <li>After running, you will have a logfile in <code>tmptmp</code> (<code>less -SR tmptmp</code> to view),
    a S3M file in <code>test.s3m</code> and a MIDI file in <code>test.mid</code>. (No logfile in case of the short version.)</li>
 <li>Listen the MIDI file and repeat steps 2-6 as needed.</li>
</ol>
The <code> >/dev/null </code> part of the command is important to prevent
your terminal being flooded with binary data. The binary data is actually
PCM data: the program will resynthesize the song into stdout. If you want
to listen it, you can try this: <code>./snd  sndfilename.snd  2>tmptmp | ecat -m</code>.
You can find the <code>ecat</code> program <a href=\"streamixer.html\">here</a>.
 <p>
The <code>2> tmptmp</code> part of the commandline instructs the shell to
redirect the <code>stderr</code> of the program into the specified file.
If the shell works properly, the program won't receive the \"2\" as a
commandline parameter.
 <p>
You can acquire <code>nezplay</code> by following these steps:
<ul>
 <li>Download <a href=\"/src/patch-nezplug-0.948-linux2-bisqwit.gz\">this patch</a> (patch-nezplug-0.948-linux2-bisqwit.gz)</li>
 <li>Download <a href=\"http://nezplug.sourceforge.net/\">Nezplug 0.9.4.8</a> source code</li>
 <li>Decompress the source code, apply the patch to the source code, compile it. Install.</li>
</ul>
 <p>
By the way... you almost certainly need an unix environment to do this all.
I guess Cygwin will do for Windows users.<br>
Nezplay may require Linux, at least
if you want it to output audible sound. It can always output to a file however.

", 'idea:1. Development ideas' => "

<ul>
 <li>Commandline options, configuration files (so that recompilation isn't needed)</li>
 <li>A library of NESified forms of each midi instrument, so that the game
  can heuristically pick the closest match without need for customization
  <ul>
   <li>Note: instruments can sometimes be NESified into two or three channels
     (for example: crystally sound is often created from 50% duty on one channel
       and 25% duty with higher octave on another channel)</li>
   <li>Note: the volume curve of instruments sometimes varies as the function of pitch</li>
  </ul></li>
 <li>Support for pitch bending (now doesn't support it, and usually confuses the pitch when it happens)</li>
</ul>

", 'copying:1. Copying' => "

sndtool2 has been written by Joel Yliluoma, a.k.a.
<a href=\"http://iki.fi/bisqwit/\">Bisqwit</a>,<br>
and is distributed under the following terms:<br>
<blockquote><code>".
str_replace("\n",'<br>',htmlspecialchars(fread(fopen('/WWW/TINYFREELICENSE','r'),filesize('/WWW/TINYFREELICENSE')))).
"</code></blockquote>

", 'req:1. Requirements' => "

GNU tools are always good to have near.

", '1. See also' => "

<ul>
 <li><a href=\"/source/opl3emu.html\">opl3emu</a> emulates the
      OPL3 chip (which is known also as the FM synthesis chip
      or AdLib). I'm investigating how OPL works so that I could
      make a good adsr-based instrument detection here.</li>
</ul>

");
include '/WWW/progdesc.php';
